#version 330 core

#define ANTI_ALIASING              // Anti-Aliasing
#define SHARPNESS 0.25             // Sharpness   [0.00 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50]
#define SATURATION 1.10            // Saturation  [0.00 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00 1.05 1.10 1.15 1.20 1.25]
#define CONTRAST 1.025             // Contrast    [0.00 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00 1.025 1.05 1.075 1.10]
#define BRIGHTNESS 0.00            // Brightness  [-1.00 -0.95 -0.90 -0.85 -0.80 -0.75 -0.70 -0.65 -0.60 -0.55 -0.50 -0.45 -0.40 -0.35 -0.30 -0.25 -0.20 -0.15 -0.10 -0.05 0.00 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00]

uniform sampler2D colortex0;
varying vec2 texcoord;

#ifdef GL_ES
precision mediump float;
#endif

varying vec2 vUv;
uniform sampler2D texture;



vec3 fxaa(sampler2D tex, vec2 uv) {
    vec2 inverseScreenSize = 1.0 / vec2(textureSize(tex, 0));

    vec3 rgbNW = texture2D(tex, uv + vec2(-1.0, -1.0) * inverseScreenSize).xyz;
    vec3 rgbNE = texture2D(tex, uv + vec2(1.0, -1.0) * inverseScreenSize).xyz;
    vec3 rgbSW = texture2D(tex, uv + vec2(-1.0, 1.0) * inverseScreenSize).xyz;
    vec3 rgbSE = texture2D(tex, uv + vec2(1.0, 1.0) * inverseScreenSize).xyz;
    vec3 rgbM  = texture2D(tex, uv).xyz;

    vec3 luma = vec3(0.299, 0.587, 0.114);
    float lumaNW = dot(rgbNW, luma);
    float lumaNE = dot(rgbNE, luma);
    float lumaSW = dot(rgbSW, luma);
    float lumaSE = dot(rgbSE, luma);
    float lumaM  = dot(rgbM,  luma);

    float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
    float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));

    vec2 dir;
    dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
    dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));

    float dirReduce = max((lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * 0.125), 0.0);
    float rcpDirMin = 1.0 / (min(abs(dir.x), abs(dir.y)) + dirReduce);
    dir = min(vec2(8.0, 8.0), max(vec2(-8.0, -8.0), dir * rcpDirMin)) * inverseScreenSize;

    vec3 rgbA = (1.0/2.0) * (
        texture2D(tex, uv + dir * (1.0 / 3.0 - 0.5)).xyz +
        texture2D(tex, uv + dir * (2.0 / 3.0 - 0.5)).xyz);

    vec3 rgbB = rgbA * (1.0/2.0) + (1.0/4.0) * (
        texture2D(tex, uv + dir * (0.0 / 3.0 - 0.5)).xyz +
        texture2D(tex, uv + dir * (3.0 / 3.0 - 0.5)).xyz);

    float lumaB = dot(rgbB, luma);

    if ((lumaB < lumaMin) || (lumaB > lumaMax)) {
        return rgbA;
    } else {
        return rgbB;
    }
}

vec3 applySharpening(vec3 color, float sharpness) {
    vec3 blurredColor = texture2D(texture, vUv).rgb;
    vec3 edge = blurredColor - color;
    return color + edge * sharpness;
}

void main() {
    vec3 sharpenedColor = texture2D(colortex0, vUv.xy).rgb;

    #ifdef ANTI_ALIASING
    vec3 antiAliasedColor = fxaa(texture, vUv);
    sharpenedColor = applySharpening(antiAliasedColor, SHARPNESS); // Adjust sharpness factor as needed
    #endif
	
    vec3 color = sharpenedColor.rgb;

    // Apply saturation
	float saturation = SATURATION;
    float average = (color.r + color.g + color.b) / 3.0;
    color.rgb = mix(vec3(average), color.rgb, saturation);

    // Apply brightness
	float brightness = BRIGHTNESS;
    color.rgb += brightness;

    // Apply contrast
	float contrast = CONTRAST;
    color.rgb = ((color.rgb - 0.5) * contrast) + 0.5;
	
	
    gl_FragColor = vec4(color, 1.0);
}
